@@ -6,11 +6,15 @@ use crate::abi::{param_type::Reader, ParamType};
 /// A field declaration inside a struct
 #[derive(Debug, Clone, PartialEq)]
 pub struct FieldDeclaration {
-    name: String,
-    ty: FieldType,
+    pub name: String,
+    pub ty: FieldType,
 }
 
 impl FieldDeclaration {
+    pub fn new(name: String, ty: FieldType) -> Self {
+        Self { name, ty }
+    }
+
     pub fn name(&self) -> &str {
         &self.name
     }
@@ -24,13 +28,11 @@ impl FieldDeclaration {
 #[derive(Debug, Clone, PartialEq)]
 pub enum FieldType {
     /// Represents elementary types, see [`ParamType`]
+    ///
+    /// Note: tuples will be treated as rust tuples
     Elementary(ParamType),
     /// A non elementary type field, treated as user defined struct
     Struct(StructFieldType),
-    // Array of user defined type
-    StructArray(StructFieldType),
-    // Array with fixed size of user defined type
-    FixedStructArray(StructFieldType, usize),
     /// Mapping
     Mapping(Box<MappingType>),
 }
@@ -42,9 +44,7 @@ impl FieldType {
 
     pub(crate) fn as_struct(&self) -> Option<&StructFieldType> {
         match self {
-            FieldType::Struct(s)
-            | FieldType::StructArray(s)
-            | FieldType::FixedStructArray(s, _) => Some(s),
+            FieldType::Struct(s) => Some(s),
             _ => None,
         }
     }
@@ -72,8 +72,8 @@ pub struct StructFieldDeclaration {
 
 /// How the type of a struct field is referenced
 #[derive(Debug, Clone, PartialEq)]
-pub struct StructFieldType {
-    /// The name of the struct
+pub struct StructType {
+    /// The name of the struct (or rather the name of the rust type)
     name: String,
     /// All previous projections up until the name
     ///
@@ -81,10 +81,63 @@ pub struct StructFieldType {
     projections: Vec<String>,
 }
 
-impl StructFieldType {
+impl StructType {
+    pub fn new(name: String, projections: Vec<String>) -> Self {
+        Self { name, projections }
+    }
+
     pub fn name(&self) -> &str {
         &self.name
     }
+}
+
+/// Represents the type of a field in a struct
+#[derive(Debug, Clone, PartialEq)]
+pub enum StructFieldType {
+    /// A non elementary type field, represents a user defined struct
+    Type(StructType),
+    // Array of user defined type
+    Array(Box<StructFieldType>),
+    // Array with fixed size of user defined type
+    FixedArray(Box<StructFieldType>, usize),
+}
+
+impl StructFieldType {
+    pub fn name(&self) -> &str {
+        match self {
+            StructFieldType::Type(ty) => &ty.name,
+            StructFieldType::Array(ty) => ty.name(),
+            StructFieldType::FixedArray(ty, _) => ty.name(),
+        }
+    }
+
+    pub fn projections(&self) -> &[String] {
+        match self {
+            StructFieldType::Type(ty) => &ty.projections,
+            StructFieldType::Array(ty) => ty.projections(),
+            StructFieldType::FixedArray(ty, _) => ty.projections(),
+        }
+    }
+
+    pub fn identifier(&self) -> String {
+        let name = self.name();
+        let path = self.projections().join(".");
+        if path.is_empty() {
+            name.to_string()
+        } else {
+            format!("{}.{}", path, name)
+        }
+    }
+
+    pub fn as_param(&self, tuple: ParamType) -> ParamType {
+        match self {
+            StructFieldType::Type(_) => tuple,
+            StructFieldType::Array(ty) => ty.as_param(ParamType::Array(Box::new(tuple))),
+            StructFieldType::FixedArray(ty, size) => {
+                ty.as_param(ParamType::FixedArray(Box::new(tuple), *size))
+            }
+        }
+    }
 
     /// Parse a struct field declaration
     ///
@@ -97,10 +150,10 @@ impl StructFieldType {
             let mut chars = input.chars();
             match chars.next() {
                 None => {
-                    return Ok(FieldType::Struct(StructFieldType {
+                    return Ok(FieldType::Struct(StructFieldType::Type(StructType {
                         name: ty,
                         projections,
-                    }))
+                    })))
                 }
                 Some(' ') | Some('\t') | Some('[') => {
                     // array
@@ -118,18 +171,23 @@ impl StructFieldType {
                                 }
                             }
                             Some(']') => {
-                                let ty = StructFieldType {
+                                let ty = StructType {
                                     name: ty,
                                     projections,
                                 };
 
                                 return if size.is_empty() {
-                                    Ok(FieldType::StructArray(ty))
+                                    Ok(FieldType::Struct(StructFieldType::Array(Box::new(
+                                        StructFieldType::Type(ty),
+                                    ))))
                                 } else {
                                     let size = size.parse().map_err(|_| {
                                         format_err!("Illegal array size `{}` at `{}`", size, input)
                                     })?;
-                                    Ok(FieldType::FixedStructArray(ty, size))
+                                    Ok(FieldType::Struct(StructFieldType::FixedArray(
+                                        Box::new(StructFieldType::Type(ty)),
+                                        size,
+                                    )))
                                 };
                             }
                             Some(c) => {
@@ -157,8 +215,8 @@ impl StructFieldType {
 /// Represents a solidity struct
 #[derive(Debug, Clone, PartialEq)]
 pub struct SolStruct {
-    name: String,
-    fields: Vec<FieldDeclaration>,
+    pub name: String,
+    pub fields: Vec<FieldDeclaration>,
 }
 
 impl SolStruct {
@@ -167,7 +225,7 @@ impl SolStruct {
     /// # Example
     ///
     /// ```
-    /// # use ethers::abi::SolStruct;
+    /// # use ethers_core::abi::SolStruct;
     /// let s = SolStruct::parse("struct MyStruct { uint x; uint y;}").unwrap();
     /// ```
     pub fn parse(s: &str) -> Result<Self> {
@@ -223,6 +281,19 @@ impl SolStruct {
     pub fn fields(&self) -> &Vec<FieldDeclaration> {
         &self.fields
     }
+
+    /// If the struct only consists of elementary fields, this will return `ParamType::Tuple` with all those fields
+    pub fn as_tuple(&self) -> Option<ParamType> {
+        let mut params = Vec::with_capacity(self.fields.len());
+        for field in self.fields() {
+            if let FieldType::Elementary(ref param) = field.ty {
+                params.push(param.clone())
+            } else {
+                return None;
+            }
+        }
+        Some(ParamType::Tuple(params))
+    }
 }
 
 /// Strips the identifier of field declaration from the input and returns it
@@ -386,10 +457,10 @@ mod tests {
                     },
                     FieldDeclaration {
                         name: "_other".to_string(),
-                        ty: FieldType::Struct(StructFieldType {
+                        ty: FieldType::Struct(StructFieldType::Type(StructType {
                             name: "Inner".to_string(),
                             projections: vec!["Some".to_string(), "Other".to_string()]
-                        }),
+                        })),
                     },
                 ],
             }
