@@ -35,7 +35,6 @@ pub use rlp;
 
 use crate::types::{Address, Bytes, U256};
 use k256::{ecdsa::SigningKey, EncodedPoint as K256PublicKey};
-use std::convert::TryInto;
 use std::ops::Neg;
 use thiserror::Error;
 
@@ -85,22 +84,41 @@ pub fn format_units<T: Into<U256>, K: Into<Units>>(amount: T, units: K) -> U256
 /// assert_eq!(eth, parse_ether(1u8).unwrap());
 /// assert_eq!(eth, parse_ether(1usize).unwrap());
 /// assert_eq!(eth, parse_ether("1").unwrap());
-pub fn parse_ether<S>(eth: S) -> Result<U256, S::Error>
+/// ```
+pub fn parse_ether<S>(eth: S) -> Result<U256, Box<dyn std::error::Error>>
 where
-    S: TryInto<U256>,
+    S: ToString,
 {
-    Ok(eth.try_into()? * WEI_IN_ETHER)
+    parse_units(eth, "ether")
 }
 
 /// Multiplies the provided amount with 10^{units} provided.
-pub fn parse_units<S, K>(amount: S, units: K) -> Result<U256, S::Error>
+///
+/// ```
+/// use ethers_core::{types::U256, utils::parse_units};
+/// let amount_in_eth = U256::from_dec_str("15230001000000000000").unwrap();
+/// let amount_in_gwei = U256::from_dec_str("15230001000").unwrap();
+/// let amount_in_wei = U256::from_dec_str("15230001000").unwrap();
+/// assert_eq!(amount_in_eth, parse_units("15.230001000000000000", "ether").unwrap());
+/// assert_eq!(amount_in_gwei, parse_units("15.230001000000000000", "gwei").unwrap());
+/// assert_eq!(amount_in_wei, parse_units("15230001000", "wei").unwrap());
+/// ```
+/// Example of trying to parse decimal WEI, which should fail, as WEI is the smallest
+/// ETH denominator. 1 ETH = 10^18 WEI.
+/// ```should_panic
+/// use ethers_core::{types::U256, utils::parse_units};
+/// let amount_in_wei = U256::from_dec_str("15230001000").unwrap();
+/// assert_eq!(amount_in_wei, parse_units("15.230001000000000000", "wei").unwrap());
+/// ```
+pub fn parse_units<K, S>(amount: S, units: K) -> Result<U256, Box<dyn std::error::Error>>
 where
-    S: TryInto<U256>,
+    S: ToString,
     K: Into<Units>,
 {
-    Ok(amount.try_into()? * 10u64.pow(units.into().as_num()))
+    let float_n: f64 = amount.to_string().parse::<f64>()? * 10u64.pow(units.into().as_num()) as f64;
+    let u256_n: U256 = U256::from_dec_str(&float_n.to_string())?;
+    Ok(u256_n)
 }
-
 /// The address for an Ethereum contract is deterministically computed from the
 /// address of its creator (sender) and how many transactions the creator has
 /// sent (nonce). The sender and nonce are RLP encoded and then hashed with Keccak-256.
@@ -117,7 +135,7 @@ pub fn get_contract_address(sender: impl Into<Address>, nonce: impl Into<U256>)
     Address::from(bytes)
 }
 
-/// Returns the CREATE2 of a smart contract as specified in
+/// Returns the CREATE2 address of a smart contract as specified in
 /// [EIP1014](https://github.com/ethereum/EIPs/blob/master/EIPS/eip-1014.md)
 ///
 /// keccak256( 0xff ++ senderAddress ++ salt ++ keccak256(init_code))[12..]
@@ -125,12 +143,71 @@ pub fn get_create2_address(
     from: impl Into<Address>,
     salt: impl Into<Bytes>,
     init_code: impl Into<Bytes>,
+) -> Address {
+    get_create2_address_from_hash(from, salt, keccak256(init_code.into().as_ref()).to_vec())
+}
+
+/// Returns the CREATE2 address of a smart contract as specified in
+/// [EIP1014](https://github.com/ethereum/EIPs/blob/master/EIPS/eip-1014.md),
+/// taking the pre-computed hash of the init code as input.
+///
+/// keccak256( 0xff ++ senderAddress ++ salt ++ keccak256(init_code))[12..]
+///
+/// # Example
+///
+/// Calculate the address of a UniswapV3 pool.
+///
+/// ```
+/// use ethers_core::{
+///     abi,
+///     abi::Token,
+///     types::{Address, Bytes, U256},
+///     utils::{get_create2_address_from_hash, keccak256},
+/// };
+///
+/// let UNISWAP_V3_POOL_INIT_CODE_HASH = Bytes::from(
+///     hex::decode("e34f199b19b2b4f47f68442619d555527d244f78a3297ea89325f843f87b8b54").unwrap(),
+/// );
+/// let factory: Address = "0x1F98431c8aD98523631AE4a59f267346ea31F984"
+///     .parse()
+///     .unwrap();
+/// let token0: Address = "0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48"
+///     .parse()
+///     .unwrap();
+/// let token1: Address = "0xc02aaa39b223fe8d0a0e5c4f27ead9083c756cc2"
+///     .parse()
+///     .unwrap();
+/// let fee = 500;
+///
+/// // abi.encode(token0 as address, token1 as address, fee as uint256)
+/// let input = abi::encode(&vec![
+///     Token::Address(token0),
+///     Token::Address(token1),
+///     Token::Uint(U256::from(fee)),
+/// ]);
+///
+/// // keccak256(abi.encode(token0, token1, fee))
+/// let salt = keccak256(&input);
+/// let pool_address =
+///     get_create2_address_from_hash(factory, salt.to_vec(), UNISWAP_V3_POOL_INIT_CODE_HASH);
+///
+/// assert_eq!(
+///     pool_address,
+///     "0x88e6A0c2dDD26FEEb64F039a2c41296FcB3f5640" // USDC/ETH pool address
+///         .parse()
+///         .unwrap()
+/// );
+/// ```
+pub fn get_create2_address_from_hash(
+    from: impl Into<Address>,
+    salt: impl Into<Bytes>,
+    init_code_hash: impl Into<Bytes>,
 ) -> Address {
     let bytes = [
         &[0xff],
         from.into().as_bytes(),
         salt.into().as_ref(),
-        &keccak256(init_code.into().as_ref()),
+        init_code_hash.into().as_ref(),
     ]
     .concat();
 
@@ -323,8 +400,20 @@ mod tests {
 
     #[test]
     fn test_parse_units() {
-        let gwei = parse_units(1, 9).unwrap();
-        assert_eq!(gwei.as_u64(), 1e9 as u64);
+        let gwei = parse_units(1.5, 9).unwrap();
+        assert_eq!(gwei.as_u64(), 15e8 as u64);
+
+        let eth_dec_float = parse_units(1.39563324, "ether").unwrap();
+        assert_eq!(
+            eth_dec_float,
+            U256::from_dec_str("1395633240000000000").unwrap()
+        );
+
+        let eth_dec_string = parse_units("1.39563324", "ether").unwrap();
+        assert_eq!(
+            eth_dec_string,
+            U256::from_dec_str("1395633240000000000").unwrap()
+        );
 
         let eth = parse_units(1, "ether").unwrap();
         assert_eq!(eth, WEI_IN_ETHER);
@@ -485,11 +574,16 @@ mod tests {
                 "E33C0C7F7df4809055C3ebA6c09CFe4BaF1BD9e0",
             ),
         ] {
+            // get_create2_address()
             let from = from.parse::<Address>().unwrap();
             let salt = hex::decode(salt).unwrap();
             let init_code = hex::decode(init_code).unwrap();
             let expected = expected.parse::<Address>().unwrap();
-            assert_eq!(expected, get_create2_address(from, salt, init_code))
+            assert_eq!(expected, get_create2_address(from, salt.clone(), init_code.clone()));
+
+            // get_create2_address_from_hash()
+            let init_code_hash = keccak256(init_code).to_vec();
+            assert_eq!(expected, get_create2_address_from_hash(from, salt, init_code_hash))
         }
     }
 
