@@ -21,13 +21,15 @@ pub enum SolcError {
     SerdeJson(#[from] serde_json::Error),
 }
 
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, Serialize, Deserialize)]
 /// The result of a solc compilation
 pub struct CompiledContract {
     /// The contract's ABI
     pub abi: Abi,
     /// The contract's bytecode
     pub bytecode: Bytes,
+    /// The contract's runtime bytecode
+    pub runtime_bytecode: Bytes,
 }
 
 /// Solidity Compiler Bindings
@@ -45,7 +47,7 @@ pub struct CompiledContract {
 /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// // Give it a glob
 /// let contracts = Solc::new("./contracts/*")
-///     .optimizer(200)
+///     .optimizer(Some(200))
 ///     .build()?;
 ///
 /// // this will return None if the specified contract did not exist in the compiled
@@ -55,21 +57,27 @@ pub struct CompiledContract {
 /// # }
 /// ```
 pub struct Solc {
+    /// The path to the Solc binary
+    pub solc_path: Option<PathBuf>,
+
     /// The path where contracts will be read from
     pub paths: Vec<String>,
 
-    /// Number of runs
-    pub optimizer: usize,
+    /// Number of optimizer runs. None for no optimization
+    pub optimizer: Option<usize>,
 
     /// Evm Version
     pub evm_version: EvmVersion,
 
     /// Paths for importing other libraries
     pub allowed_paths: Vec<PathBuf>,
+
+    /// Additional arguments to pass to solc
+    pub args: Vec<String>,
 }
 
 impl Solc {
-    /// Instantiates the Solc builder for the provided paths
+    /// Instantiates The Solc builder with the provided glob of Solidity files
     pub fn new(path: &str) -> Self {
         // Convert the glob to a vector of string paths
         // TODO: This might not be the most robust way to do this
@@ -77,24 +85,46 @@ impl Solc {
             .expect("could not get glob")
             .map(|path| path.expect("path not found").to_string_lossy().to_string())
             .collect::<Vec<String>>();
+        Self::new_with_paths(paths)
+    }
 
+    /// Instantiates the Solc builder for the provided paths
+    pub fn new_with_paths(paths: Vec<String>) -> Self {
         Self {
             paths,
-            optimizer: 200, // default optimizer runs = 200
+            solc_path: None,
+            optimizer: Some(200), // default optimizer runs = 200
             evm_version: EvmVersion::Istanbul,
             allowed_paths: Vec::new(),
+            args: Vec::new(),
         }
     }
 
     /// Gets the ABI for the contracts
     pub fn build_raw(self) -> Result<HashMap<String, CompiledContractStr>> {
-        let mut command = Command::new(SOLC);
+        let path = self.solc_path.unwrap_or_else(|| PathBuf::from(SOLC));
 
-        command
-            .arg("--evm-version")
-            .arg(self.evm_version.to_string())
-            .arg("--combined-json")
-            .arg("abi,bin");
+        let mut command = Command::new(&path);
+        let version = Solc::version(Some(path));
+
+        command.arg("--combined-json").arg("abi,bin,bin-runtime");
+
+        if (version.starts_with("0.5") && self.evm_version < EvmVersion::Istanbul)
+            || !version.starts_with("0.4")
+        {
+            command
+                .arg("--evm-version")
+                .arg(self.evm_version.to_string());
+        }
+
+        if let Some(runs) = self.optimizer {
+            command
+                .arg("--optimize")
+                .arg("--optimize-runs")
+                .arg(runs.to_string());
+        }
+
+        command.args(self.args);
 
         for path in self.paths {
             command.arg(path);
@@ -135,7 +165,21 @@ impl Solc {
                         )))
                     }
                 };
-                contracts.insert(name, CompiledContractStr { abi, bin });
+
+                let runtime_bin =
+                    if let serde_json::Value::String(bin) = contract["bin-runtime"].take() {
+                        bin
+                    } else {
+                        panic!("no runtime bytecode found")
+                    };
+                contracts.insert(
+                    name,
+                    CompiledContractStr {
+                        abi,
+                        bin,
+                        runtime_bin,
+                    },
+                );
             } else {
                 return Err(SolcError::SolcError(
                     "could not find `bin` in solc output".to_string(),
@@ -161,7 +205,19 @@ impl Solc {
                 let bytecode = hex::decode(contract.bin)
                     .expect("solc did not produce valid bytecode")
                     .into();
-                (name, CompiledContract { abi, bytecode })
+
+                // parse the runtime bytecode
+                let runtime_bytecode = hex::decode(contract.runtime_bin)
+                    .expect("solc did not produce valid runtime-bytecode")
+                    .into();
+                (
+                    name,
+                    CompiledContract {
+                        abi,
+                        bytecode,
+                        runtime_bytecode,
+                    },
+                )
             })
             .collect::<HashMap<String, CompiledContract>>();
 
@@ -172,12 +228,13 @@ impl Solc {
     ///
     /// # Panics
     ///
-    /// If `solc` is not in the user's $PATH
-    pub fn version() -> String {
-        let command_output = Command::new(SOLC)
+    /// If `solc` is not found
+    pub fn version(solc_path: Option<PathBuf>) -> String {
+        let solc_path = solc_path.unwrap_or_else(|| PathBuf::from(SOLC));
+        let command_output = Command::new(&solc_path)
             .arg("--version")
             .output()
-            .unwrap_or_else(|_| panic!("`{}` not in user's $PATH", SOLC));
+            .unwrap_or_else(|_| panic!("`{:?}` not found", solc_path));
 
         let version = command_output
             .stdout
@@ -196,8 +253,30 @@ impl Solc {
         self
     }
 
-    /// Sets the optimizer runs (default = 200)
-    pub fn optimizer(mut self, runs: usize) -> Self {
+    /// Sets the path to the solc binary
+    pub fn solc_path(mut self, path: PathBuf) -> Self {
+        self.solc_path = Some(std::fs::canonicalize(path).unwrap());
+        self
+    }
+
+    /// Sets the optimizer runs (default = 200). None indicates no optimization
+    ///
+    /// ```rust,no_run
+    /// use ethers_core::utils::Solc;
+    ///
+    /// // No optimization
+    /// let contracts = Solc::new("./contracts/*")
+    ///     .optimizer(None)
+    ///     .build().unwrap();
+    ///
+    /// // Some(200) is default, optimizer on with 200 runs
+    /// // .arg() allows passing arbitrary args to solc command
+    /// let optimized_contracts = Solc::new("./contracts/*")
+    ///     .optimizer(Some(200))
+    ///     .arg("--metadata-hash=none")
+    ///     .build().unwrap();
+    /// ```
+    pub fn optimizer(mut self, runs: Option<usize>) -> Self {
         self.optimizer = runs;
         self
     }
@@ -208,9 +287,27 @@ impl Solc {
         self.allowed_paths = paths;
         self
     }
+
+    /// Adds an argument to pass to solc
+    pub fn arg<T: Into<String>>(mut self, arg: T) -> Self {
+        self.args.push(arg.into());
+        self
+    }
+
+    /// Adds multiple arguments to pass to solc
+    pub fn args<I, S>(mut self, args: I) -> Self
+    where
+        I: IntoIterator<Item = S>,
+        S: Into<String>,
+    {
+        for arg in args {
+            self = self.arg(arg);
+        }
+        self
+    }
 }
 
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
 pub enum EvmVersion {
     Homestead,
     TangerineWhistle,
@@ -250,4 +347,6 @@ pub struct CompiledContractStr {
     pub abi: String,
     /// The contract's bytecode in hex
     pub bin: String,
+    /// The contract's runtime bytecode in hex
+    pub runtime_bin: String,
 }
